EmergeRT = LibStub("AceAddon-3.0"):NewAddon("EmergeRT", "AceConsole-3.0", "AceEvent-3.0")

local AceGUI = LibStub("AceGUI-3.0")
local BI = LibStub("LibBossIDs-1.0").BossIDs
local LDBIcon = LibStub("LibDBIcon-1.0", true)
local LDB = LibStub("LibDataBroker-1.1", true)

local selectedEvent = nil
local selectedOverview = nil
local selectedItem = nil
local selectedJoin = nil
local selectedLeave = nil
local selectedBoss = nil
local selectedWipe = nil
local selectedPlayer = nil

local difficulties = {"10","25"}
local lastChecked = 0
local checkPending = nil
local lastWipe = 0

local isDst = false

local itemID = nil
local LinkBox = nil

Orig_ChatEdit_InsertLink = ChatEdit_InsertLink
function ChatEdit_InsertLink(text)
  if text and LinkBox and LinkBox:IsVisible() then
    LinkBox:SetText(text)
	itemID = text
    return true
  else
    Orig_ChatEdit_InsertLink(text)
  end
end

-- Those are ignored (Marks, Emblems, Gems, etc.)
local ignoredItems = {
	[40752] = true, -- Emblem of Heroism
	[40753] = true, -- Emblem of Valor
	[45624] = true, -- Emblem of Conquest
	[47241] = true, -- Emblem of Triumph
	[34057] = true -- Abyss Crystal
	
}

-- Ignored boss mobs
local ignoredBosses = {
	[30452]= true,	-- Tenebron
	[30451]= true,	-- Shadron
	[30449]= true,	-- Vesperon
	[15929]= true,	-- Stalagg
	[15930]= true,	-- Feugen
	[32882]= true,	-- Jormungar Behemoth
	[33329]= true, -- Heart of the Deconstructor
	[32933]= true, -- Kologarns left arm
	[32934]= true, -- Kologarns right arm
	[34035]= true, -- Feral Defender
	[34014]= true, -- Sanctum Sentry
	[32915]= true, -- Elder Brightleaf
	[32913]= true, -- Elder Ironbranch
	[32914]= true, -- Elder Stonebark
	[33432]= true, -- Leviathan MKII
	[33651]= true, -- VX-001
	[33670]= true, -- Aerial Command Unit
	[33136]= true, -- Guardian of Yogg-Saron
	[34799]= true, -- Jormungars: Dreadscale
	[35144]= true, -- Jormungars: Acidmaw
	[34796] = true, -- Gormok the Impaler
	[37025]= true, -- Stinky
	[37217] = true, -- Precious
}

local multiBosses = {
	["Four Horsemen"] = {
		[30549]=true,	-- Baron Rivendare
		[16063]=true,	-- Sir Zeliek
		[16065]=true,	-- Lady Blaumeux
		[16064]=true,	-- Thane Korth'azz
		alive = 4
	},
	["Iron Council"] = {
		[32857]=true,	-- Stormcaller Brundir
		[32867]=true,	-- Steelbreaker
		[32927]=true,	-- Runemaster Molgeim
		alive = 3
	},
	["Faction Champions"] = {
		[34466] = true, -- Anthar Forgemender
		[34472] = true, -- Irieth Shadowstep
		[34469] = true, -- Melador Valestrider
		[34475] = true, -- Shocuul
		[34463] = true, -- Shaabad
		[34461] = true, -- Tyrius Duskblade
		[34465] = true, -- Velanaa
		[34474] = true, -- Serissa Grimdabbler
		[35465] = true, -- Zhaagrym
		[34467] = true, -- Alyssia Moonstalker
		[35610] = true, -- Cat 
		[34473] = true, -- Brienna Nightfell
		alive = 12
	}
}

local bossYells = {
	["Freya"] = "His hold on me dissipates. I can see clearly once more. Thank you, heroes.",
	["Hodir"] = "I... I am released from his grasp... at last.",
	["Thorim"] = "Stay your arms! I yield!",
	["Mimiron"] = "It would appear that I've made a slight miscalculation.",
	["Icecrown Gunship Battle"] = "The Alliance falter. Onward to the Lich King!",
	["Icecrown Gunship Battle"] = "Don't say I didn't warn ya, scoundrels! Onward, brothers and sisters!"
}

EmergeRT.options = {
	desc = "Emerge Raid Tracker",
	type = "group",
	args = {
		show = {
			name = "show",
			desc = "Show the EmergeRT window",
			type = "execute",
			func = function()
				EmergeRT:ShowGUI()
			end,
		},
		clear = {
			name = "clear",
			desc = "Clear all data",
			type = "execute",
			func = function()
				EmergeRT:ClearCheck()
			end,
		},
		difficulty = {
			name = "difficulty",
			desc = "Needed dungeon difficulty (10/25)",
			type = "select",
			values = difficulties,
			get = function(info) return EmergeRT.db.profile.difficulty end,
			set = function(info,val)
								EmergeRT.db.profile.difficulty = val
								EmergeRT:Print("Difficulty set to: "..difficulties[val])
								EmergeRT:RAID_ROSTER_UPDATE()
			end,
		},
		members = {
			name = "members",
			desc = "Minimum Number of Raidmembers",
			type = "input",
			set = function(info,val)
						EmergeRT.db.profile.members = tonumber(val)
						EmergeRT:Print("Members set to: "..tostring(val))
						EmergeRT:RAID_ROSTER_UPDATE()
			end
		},
		minimap = {
			name = "minimap",
			desc = "Toggle Minimap Icon",
			type = "toggle",
			get = function(info) return EmergeRT.db.profile.minimap.show end,
			set = function(info,val)
						if val then
							LDBIcon:Show("EmergeRT")
						else
							LDBIcon:Hide("EmergeRT")
						end
			end,
		},
		popup = {
			name = "popup",
			desc = "Toggle Loot-DKP Popup",
			type = "toggle",
			get = function(info) return EmergeRT.db.profile.popup end,
			set = function(info,val) EmergeRT.db.profile.popup = val EmergeRT:Print("Popup set to: "..tostring(val)) end,
		},
		standby = {
			type = "toggle",
			name = "Standby",
			desc = "Toggles EmergeRT on/off",
			get = function(info) return EmergeRT.db.profile.standby end,
			set = function(info, val)
				EmergeRT.db.profile.standby = val
				if val then
					EmergeRT:Disable()
				else
					EmergeRT:Enable()					
				end
			end,
		},
		status = {
			name = "status",
			desc = "Print current Tracking status",
			type = "execute",
			func = function()
				EmergeRT:PrintStatus()
			end,
		},
		wipe = {
			name = "wipe",
			desc = "Toggle WipeCheck",
			type = "toggle",
			get = function(info) return EmergeRT.db.profile.wipe end,
			set = function(info,val)
					EmergeRT.db.profile.wipe = val
					if val and EmergeRT.db.profile.raidActive then
						EmergeRT:RegisterEvent("UNIT_HEALTH")
					else
						EmergeRT:UnregisterEvent("UNIT_HEALTH")
					end
					EmergeRT:Print("WipeCheck set to: "..tostring(val))
					end,
		},
	},
}

local defaults = {
	profile = {
		events = {},
		popup = true,
		difficulty = 2,
		raidActive = nil,
		members = 11,
		standby = false,
		minimap = {},
		attendees = {}
	}
}

function EmergeRT:OnInitialize()
	self.db = LibStub("AceDB-3.0"):New("EmergeRTDB", defaults, "profile")
	LibStub("AceConfig-3.0"):RegisterOptionsTable("EmergeRT", self.options)
	LibStub("AceConfigCmd-3.0"):CreateChatCommand("emergert", "EmergeRT")
	LibStub("AceConfigCmd-3.0"):CreateChatCommand("ert", "EmergeRT")
	LibStub("AceConfigCmd-3.0"):CreateChatCommand("ERT", "EmergeRT")
	
	if LDB then
		self.ldb = LDB:NewDataObject("EmergeRT",
				{
					type =  "launcher", 
					icon = "Interface\\Addons\\EmergeRT\\icon",
					OnClick = function(clickedframe, button)
						if button == "LeftButton" then
							self:ShowGUI()
						elseif button == "RightButton" then
							self:ClearCheck()
						end
					end,
					OnTooltipShow = function(tooltip)
						tooltip:AddLine("EmergeRT")
						tooltip:AddLine("|cffffff00Left click|r to open the GUI.")
						tooltip:AddLine("|cffffff00Right click|r to clear tracked data.")
					end,
				})
		if LDBIcon then
			LDBIcon:Register("EmergeRT", self.ldb, self.db.profile.minimap)
			if not self.db.profile.minimap.show then
				LDBIcon:Hide("EmergeRT")
			end
		end
	end
	
	if self.db.profile.standby then
		self:Disable()
	end
end

function EmergeRT:OnEnable()
	self:Print("is now enabled")
	self.multiBosses = multiBosses
	if self.db.profile.raidActive then
		self:RegisterEvent("COMBAT_LOG_EVENT_UNFILTERED")
		self:RegisterEvent("CHAT_MSG_LOOT")
		self:RegisterEvent("CHAT_MSG_MONSTER_YELL")
		if self.db.profile.wipe then
			self:RegisterEvent("UNIT_HEALTH")
		end
		self:Print("is still tracking!")
	end
	self:RegisterEvent("RAID_ROSTER_UPDATE")
end

function EmergeRT:OnDisable()
	self:UnregisterAllEvents()
	self.db.profile.raidActive = nil
	self:Print("is now disabled")
end

function EmergeRT:PrintStatus()
	if self.db.profile.raidActive then
		self:Print("is tracking.")
	else
		self:Print("is not tracking.")
	end
	self:Print("Popup: "..tostring(self.db.profile.popup))
	self:Print("Difficulty: "..difficulties[self.db.profile.difficulty])
	self:Print("Members: "..self.db.profile.members)
	self:Print("Standby: "..tostring(self.db.profile.standby))
	self:Print("Minimap: "..tostring(self.db.profile.minimap.show))
end

function EmergeRT:RAID_ROSTER_UPDATE()
	local num = GetNumRaidMembers()
	
	local id, inBG = false
	for i=1, MAX_BATTLEFIELD_QUEUES do
		_, _, id = GetBattlefieldStatus(i);
		if( id > 0 ) then
			inBG = true
		end
	end

	if not inBG then
		if (num < self.db.profile.members or GetRaidDifficulty() ~= self.db.profile.difficulty) and self.db.profile.raidActive then
			self.db.profile.raidActive = nil
			self:Print("Tracking deactivated.")
			self:UnregisterEvent("COMBAT_LOG_EVENT_UNFILTERED")
			self:UnregisterEvent("CHAT_MSG_LOOT")
			self:UnregisterEvent("CHAT_MSG_MONSTER_YELL")
			self.db.profile.events[#self.db.profile.events].endTime = time()
			self.db.profile.attendees = {}
			for i,v in ipairs(self.db.profile.events[#self.db.profile.events].attendees) do
				local joins = 0
				for u,j in ipairs(self.db.profile.events[#self.db.profile.events].joins) do
					if j.name == v.name then
						joins = joins + 1
					end
				end
				local leaves = 0
				for u,l in ipairs(self.db.profile.events[#self.db.profile.events].leaves) do
					if l.name == v.name then
						leaves = leaves + 1
					end
				end
				if joins > leaves then
					table.insert(self.db.profile.events[#self.db.profile.events].leaves, {name=v.name, time=self.db.profile.events[#self.db.profile.events].endTime})
				end
			end
		elseif GetRaidDifficulty() == self.db.profile.difficulty then
			if not self.db.profile.raidActive and num >= self.db.profile.members then
				self.db.profile.raidActive = true
				self:Print("Tracking activated.")
				self:RegisterEvent("COMBAT_LOG_EVENT_UNFILTERED")
				self:RegisterEvent("CHAT_MSG_LOOT")
				self:RegisterEvent("CHAT_MSG_MONSTER_YELL")
				if self.db.profile.wipe then
					self:RegisterEvent("UNIT_HEALTH")
				end
				self:AddEvent()
				for i=1, num do
					local name = GetRaidRosterInfo(i)
					local found = false
					for u, s in ipairs(self.db.profile.attendees) do
						if s.name == name then
							found = true
							break
						end
					end
					if not found then
						local name,_,_,level,class = GetRaidRosterInfo(i)
						local race = UnitRace("raid"..i)
						table.insert(self.db.profile.events[#self.db.profile.events].joins, {['name']=name, ['time']=time()})
						table.insert(self.db.profile.attendees, {['name']=name, ['level']=level, ['class']=class, ['race']=race})
						table.insert(self.db.profile.events[#self.db.profile.events].attendees, {['name']=name, ['level']=level, ['class']=class, ['race']=race})
					end
				end
			elseif self.db.profile.raidActive and num >= self.db.profile.members then
				local currentAttendees={}
				for i=1, num do
					local name,_,_,level,class = GetRaidRosterInfo(i)
					local race = UnitRace("raid"..i)
					table.insert(currentAttendees, {['name']=name, ['level']=level, ['class']=class, ['race']=race})
				end
				for i,c in ipairs(currentAttendees) do
					local found = false
					for u, s in ipairs(self.db.profile.attendees) do
						if s.name == c.name then
							found = true
							break
						end
					end
					if not found then
						table.insert(self.db.profile.events[#self.db.profile.events].joins, {['name']=c.name,['time']=time()})
						table.insert(self.db.profile.attendees, c)
						local found = false
						for u, s in ipairs(self.db.profile.events[#self.db.profile.events].attendees) do
							if s.name == c.name then
								found = true
								break
							end
						end
						if not found then
							table.insert(self.db.profile.events[#self.db.profile.events].attendees, c)
						end
					end
				end
				for i,s in ipairs(self.db.profile.attendees) do
					local found = false
					for u, c in ipairs(currentAttendees) do
						if s.name == c.name then
							found = true
							break
						end
					end
					if not found then
						table.insert(self.db.profile.events[#self.db.profile.events].leaves, {['name']=s.name,['time']=time()})
						table.remove(self.db.profile.attendees, i)
					end
				end
			end
		end
	end
end

local function TimeAsUnix(timestr)
	if not timestr then return end
	local datestr, timestr = strsplit(" ", timestr)
	local month, day, year = strsplit("/", datestr)
	local hour, min, sec = strsplit(":", timestr)
	return time({month=month, day=day, year=year, hour=hour, min=min, sec=sec, isdst=isDst})
end

local function FromUnixTime(timestamp)
	if not timestamp then return end
	local dt = date("*t", tonumber(timestamp))
	isDst = dt.isdst
	return string.format("%02d/%02d/%d %02d:%02d:%02d", dt.month, dt.day, dt.year, dt.hour, dt.min, dt.sec)
end

function EmergeRT:AddEvent()	
	local event = {}
	event.key = FromUnixTime(time())
	event.realm = GetRealmName()
	event.zone = GetRealZoneText()
	event.note = nil
	event.startTime = time()
	event.endTime = nil
	event.difficulty = GetRaidDifficulty()
	event.attendees = {}
	event.loots = {}
	event.bosses = {}
	event.wipes = {}
	event.joins = {}
	event.leaves = {}
	event.dkpSet = ""
	table.insert(self.db.profile.events, event)
end

function EmergeRT:AddBoss(mob)
	local boss = {
		name = mob,
		time = time(),
		attendees = self.db.profile.attendees
	}
	
	table.insert(self.db.profile.events[#self.db.profile.events].bosses, boss)
end

function EmergeRT:AddWipe()
	table.insert(self.db.profile.events[#self.db.profile.events].wipes, {['time']=time()})
	lastWipe = time()
	self.multiBosses = multiBosses
end

function EmergeRT:UNIT_HEALTH()
	local t = time()
	if difftime(t, lastWipe) > 60 and difftime(t,lastChecked) > 0.5 and not checkPending then
		lastChecked = t
		local membercount = GetNumRaidMembers();
		local memberdead = 0;
		for i = 1, membercount do
			if (UnitIsDeadOrGhost("raid"..i)) then
				memberdead = memberdead + 1;
			end;
		end;
		if memberdead == membercount then
			self:AddWipe()
		elseif memberdead/membercount > 0.8 then
			self:IsWipe()
			checkPending = true
		end
	end
end

local linkpat = '|c........|Hitem:(%%d+):.-|h|r'
local globalpatterns = {
   "LOOT_ITEM",
   "LOOT_ITEM_SELF",
}
local patterns = {}
for _, global in ipairs(globalpatterns) do
   local pattern = _G[global]
   if strsub(pattern, 1,2) == "%s" then
      pattern = "([^ ]+)"..strsub(pattern, 3)
   end
   patterns[global] = gsub(gsub(pattern, "%%d", "(%%d+)"), "%%s", linkpat)
end

-- Check for Loot
function EmergeRT:CHAT_MSG_LOOT(_,msg)
   player, item = strmatch(msg, patterns["LOOT_ITEM"])
   if player and item then
      self:ItemLooted(player, item)
      return
   end

   item = strmatch(msg, patterns["LOOT_ITEM_SELF"])
   if item then
      self:ItemLooted(UnitName("player"), item)
      return
   end
end

-- Check for Bossdefeat
function EmergeRT:CHAT_MSG_MONSTER_YELL(_,msg)
	local found = false
	local boss = nil
	for i,v in ipairs(bossYells) do
		if msg:find(v) then
			found = true
			boss = i
		end
	end
	if found then
		self:Print(format("%s down. Entry added!", boss))
		self:AddBoss(boss)
	end
end

-- Check for Bosskill
function EmergeRT:COMBAT_LOG_EVENT_UNFILTERED(_, _, event, _, _, _, guid, mob)
	guid = tonumber((guid):sub(-12,-7),16)
	if event ~= "UNIT_DIED" and event ~= "UNIT_DESTROYED" then return end
	if mob and not ignoredBosses[guid] and BI[guid] then
		local found = false
		for i,v in pairs(self.multiBosses) do
			if v[guid] then
				found = true
				v.alive = v.alive - 1
				if v.alive == 0 then
					self:AddBoss(i)
					self:Print(format("%s down. Entry added!", i))
				else
					self:Print(format("%s down, %d more to go!", mob, v.alive))
				end
				break
			end
		end
		
		if not found then
			self:Print(format("%s down. Entry added!", mob))
			self:AddBoss(mob)
		end
	end
end

-- Clear all Data
function EmergeRT:ClearAll()
	self.db.profile.raidActive = nil
	self.db.profile.attendees = {}
	self.db.profile.events = {}
	
	self.multiBosses = multiBosses
	
	selectedEvent=nil
	selectedOverview=nil
	selectedItem=nil
	
	
	self:Print("All data cleared!")
end

-- Get Item and save into DB
function EmergeRT:ItemLooted(player, item)
	local _, itemLink, itemRarity = GetItemInfo(item)
	
	local itemID = select(3, itemLink:find("item:(%d+):"))
	if not itemID then return false end
	itemID = tonumber(itemID:trim())
	if type(itemID) ~= "number" then return false end
	if ignoredItems[itemID] or itemRarity < 4  then return false end
	
	GameTooltip:SetHyperlink(itemLink)
	
	local loot = {
		player = player,
		item = itemID,
		dkp = 0,
		time = time(),
		boss = nil,
		difficulty = GetRaidDifficulty(),
		zone = GetRealZoneText(),
		note = nil
	}
	
	if #self.db.profile.events[#self.db.profile.events].bosses > 0 then
		if difftime(loot.time,self.db.profile.events[#self.db.profile.events].bosses[#self.db.profile.events[#self.db.profile.events].bosses].time) < 300 then
			loot.boss = self.db.profile.events[#self.db.profile.events].bosses[#self.db.profile.events[#self.db.profile.events].bosses].name
		else
			loot.boss = "Trash"
		end
	else
		loot.boss = "Trash"
	end
	
	table.insert(self.db.profile.events[#self.db.profile.events].loots, loot)
	
	if self.db.profile.popup then
		self:GetDKP()
	end
end

-- Convert selected kill to EQDKP-String
function EmergeRT:CreateExportString(event)
	if not event.endTime then return "Event has no Endtime!" end
	local text = "<RaidInfo><Version>1.4</Version>"

	text = text .. "<key>" .. event.key .. "</key>"
	text = text .. "<realm>" .. event.realm .. "</realm>"
	text = text .. "<start>" .. FromUnixTime(event.startTime) .. "</start>"
	text = text .. "<end>" .. FromUnixTime(event.endTime) .. "</end>"
	text = text .. "<zone>" .. event.zone .. "</zone>"
	text = text .. "<difficulty>" .. event.difficulty .. "</difficulty>"
	text = text .. "<note><![CDATA[" .. (event.note or "") .. " - Zone: " ..  event.zone .. "]]></note>"

	text = text .. "<exporter>" .. UnitName("Player") .. "</exporter>"

	text = text .. "<PlayerInfos>"
	
	for i, v in ipairs(event.attendees) do
		text = text .. "<key"..i.."><name>" .. v.name .. "</name>"
		if v.class then text = text .. "<class>" .. v.class .. "</class>" end
		if v.level then text = text .. "<level>" .. v.level .. "</level>" end
		if v.race then text = text .. "<race>" .. v.race .. "</race>" end
		if v.note then text = text .. "<note><![CDATA["..v.note.."]]></note>" end
		text = text .. "</key"..i..">"
	end

	text = text .. "</PlayerInfos>"

	text = text .. "<BossKills>"
	for i,boss in ipairs(event.bosses) do
		text = text .. "<key"..i.."><name>" .. boss.name .. "</name><time>" .. FromUnixTime(boss.time) .. "</time>"
		--[[
		text = text .. "<attendees>"
		for k,v in ipairs(boss.attendees) do
			text = text .. "<key"..k.."><name>"..v.name.."</name></key"..k..">"
		end
		text = text .. "</attendees>"]]--
		text = text .. "</key"..i..">"
	end
	text = text .. "</BossKills>"
	
	text = text .. "<Wipes>"
	for i,wipe in ipairs(event.wipes) do
		text = text .. "<Wipe>" .. wipe.time .. "</Wipe>"
	end
	text = text .. "</Wipes>"
	
	local players_join = ""
	local players_leave = ""
	
	for i,join in ipairs(event.joins) do
		players_join = players_join .. "<key"..i.."><player>" .. join.name .. "</player><time>" .. FromUnixTime(join.time) .."</time></key"..i..">"
	end
	for i,leave in ipairs(event.leaves) do
		players_leave = players_leave .. "<key"..i.."><player>" .. leave.name .. "</player><time>" .. FromUnixTime(leave.time) .."</time></key"..i..">"
	end
	
	text = text .. "<Join>" .. players_join .. "</Join>"
	text = text .. "<Leave>" .. players_leave .. "</Leave>"

	text = text .. "<Loot>"


	for i, item in ipairs(event.loots) do
		local itemName,_,rarity = GetItemInfo(item.item)
		local _,_,_,hex = GetItemQualityColor(rarity)

		text = text .. "<key"..i..">"
		text = text .. "<ItemName>" .. (itemName or "?") .. "</ItemName>"
		text = text .. "<ItemID>" .. item.item .. "</ItemID>"
		text = text .. "<Count>1</Count>"
		text = text .. "<Color>" .. strsub(hex,3) .. "</Color>"
		text = text .. "<Player>" .. item.player .. "</Player>"
		text = text .. "<Costs>" .. item.dkp .. "</Costs>"
		text = text .. "<Time>" .. FromUnixTime(item.time) .. "</Time>"
		text = text .. "<Zone>" .. item.zone .. "</Zone>"
		text = text .. "<Boss>" .. item.boss .. "</Boss>"
		text = text .. "<Difficulty>" .. item.difficulty .. "</Difficulty>"
		text = text .. "<Note><![CDATA[" .. (item.note or "") .. " - Zone: " .. item.zone .. " ("..difficulties[tonumber(item.difficulty)]..") - Boss: " .. item.boss .. " - " .. item.dkp .." DKP]]></Note>"
		text = text .. "</key"..i..">"
	end

	text = text .. "</Loot>"
	text = text .. "</RaidInfo>"
	return text
end

function EmergeRT:ShowGUI()
	if self.frame and self.frame:IsShown() then
		AceGUI:Release(self.frame)
		self.frame = nil
	end

	local fMain = AceGUI:Create("Frame")
	fMain:SetTitle("EmergeRT Menu")
	fMain:SetLayout("List")
	fMain:SetWidth(800)
	fMain:SetHeight(600)
	fMain:SetCallback("OnClose", function(widget) AceGUI:Release(widget) self.frame = nil end)
	fMain:Show()
	
	self.frame = fMain
	
	local tabEvents = AceGUI:Create("TabGroup")
	tabEvents:SetLayout("Fill")
	tabEvents:SetFullWidth(true)
	tabEvents:SetHeight(400)
	local tabs = {}
	for i,v in ipairs(self.db.profile.events) do
		table.insert(tabs, {value = i, text = v.key})
	end
	tabEvents:SetTabs(tabs)
	tabEvents:SetCallback("OnGroupSelected", function(widget,event,value) self:SelectEvent(widget,value) end )
	
	local bNew = AceGUI:Create("Button")
	bNew:SetText("Create new Event")
	bNew:SetWidth(200)
	bNew:SetCallback("OnClick", function()
									self:AddEvent()
									self:ShowGUI()
								end)
	
	fMain:SetStatusText(#tabs .." entries found.")

	fMain:AddChildren(bNew, tabEvents)
	tabEvents:SelectTab(selectedEvent)
end

function EmergeRT:DrawOverview(widget)
	local tabOverview = AceGUI:Create("TabGroup")
	tabOverview:SetLayout("Fill")
	local tabs = {{value="Event", text="Event"},{value="Players", text="Players"},{value="Joins", text="Joins"},{value="Leaves", text="Leaves"},{value="Bosses", text="Bosses"},{value="Wipes", text="Wipes"},{value="Loots", text="Loots"}}
	tabOverview:SetTabs(tabs)
	tabOverview:SetCallback("OnGroupSelected", function(w,e,v) self:SelectOverview(w,e,v) end )
	
	widget:AddChild(tabOverview)
	tabOverview:SelectTab(selectedOverview)
end

function EmergeRT:SelectEvent(widget, value)
	widget:ReleaseChildren()
	if selectedEvent ~= value then
		selectedEvent = value
		selectedOverview = nil
		selectedPlayer = nil
		selectedJoin = nil
		selectedLeave = nil
		selectedBoss = nil
		selectedWipe = nil
		selectedItem = nil
	end
	self:DrawOverview(widget)
end

function EmergeRT:DrawEvent(widget)
	local sfEvent = AceGUI:Create("ScrollFrame")
	sfEvent:SetLayout("List")
	
	local sgRow1 = AceGUI:Create("SimpleGroup")
	sgRow1:SetLayout("Flow")
	sgRow1:SetFullWidth(true)
	
	local eName = AceGUI:Create("EditBox")
	eName:SetLabel("Key")
	eName:SetWidth(180)
	eName:SetText(self.db.profile.events[selectedEvent].key)
	eName:SetCallback("OnEnterPressed",function(widget,event,text) self.db.profile.events[selectedEvent].key = strtrim(text) self:ShowGUI() end )
	
	local eZone = AceGUI:Create("EditBox")
	eZone:SetLabel("Zone")
	eZone:SetWidth(180)
	eZone:SetText(self.db.profile.events[selectedEvent].zone)
	eZone:SetCallback("OnEnterPressed",function(widget,event,text) widget.editbox:ClearFocus() self.db.profile.events[selectedEvent].zone = strtrim(text) end )
	
	local sgRow2 = AceGUI:Create("SimpleGroup")
	sgRow2:SetLayout("Flow")
	sgRow2:SetFullWidth(true)
	
	local eStart = AceGUI:Create("EditBox")
	eStart:SetLabel("Starttime")
	eStart:SetWidth(180)
	eStart:SetText(FromUnixTime(self.db.profile.events[selectedEvent].startTime))
	eStart:SetCallback("OnEnterPressed",function(widget,event,text) widget.editbox:ClearFocus() self.db.profile.events[selectedEvent].startTime = TimeAsUnix(strtrim(text)) end )
		
	local eEnd = AceGUI:Create("EditBox")
	eEnd:SetLabel("Endtime")
	eEnd:SetWidth(180)
	eEnd:SetText(FromUnixTime(self.db.profile.events[selectedEvent].endTime))
	eEnd:SetCallback("OnEnterPressed",function(widget,event,text) widget.editbox:ClearFocus() self.db.profile.events[selectedEvent].endTime = TimeAsUnix(strtrim(text)) end )
	
	local sgRow3 = AceGUI:Create("SimpleGroup")
	sgRow3:SetLayout("Flow")
	sgRow3:SetFullWidth(true)
	
	local bStart = AceGUI:Create("Button")
	bStart:SetText("Current Time")
	bStart:SetWidth(180)
	bStart:SetCallback("OnClick", function() local t = time() eStart:SetText(FromUnixTime(t)) self.db.profile.events[selectedEvent].startTime = t end)
	
	local bEnd = AceGUI:Create("Button")
	bEnd:SetText("Current Time")
	bEnd:SetWidth(180)
	bEnd:SetCallback("OnClick", function() local t = time() eEnd:SetText(FromUnixTime(t)) self.db.profile.events[selectedEvent].endTime = t end)
	
	local eDKPSet = AceGUI:Create("EditBox")
	eDKPSet:SetLabel("DKP Set")
	eDKPSet:SetWidth(180)
	eDKPSet:SetText(self.db.profile.events[selectedEvent].dkpSet)
	eDKPSet:SetCallback("OnEnterPressed",function(widget,event,text) widget.editbox:ClearFocus() self.db.profile.events[selectedEvent].dkpSet = strtrim(text) end )
	
	local sgControls = AceGUI:Create("SimpleGroup")
	sgControls:SetLayout("Flow")
	sgControls:SetFullWidth(true)
	
	local meExport = AceGUI:Create("MultiLineEditBox")
	meExport:SetLabel("Export-String")
	meExport:SetFullWidth(true)
	meExport:SetText()
	
	local bExport = AceGUI:Create("Button")
	bExport:SetText("Export Event")
	bExport:SetWidth(180)
	bExport:SetCallback("OnClick", function()
										meExport:SetText(self:CreateExportString(self.db.profile.events[selectedEvent]))
									end)
	
	local bRemove = AceGUI:Create("Button")
	bRemove:SetText("Remove Event")
	bRemove:SetWidth(180)
	bRemove:SetCallback("OnClick", function()
										table.remove(self.db.profile.events, selectedEvent)
										selectedEvent = nil
										selectedOverview = nil
										self:ShowGUI()
									end)
									
	widget:AddChild(sfEvent)
	sfEvent:AddChildren(sgRow1, sgRow2, sgRow3, eDKPSet, sgControls, meExport)
	sgRow1:AddChildren(eName, eZone)
	sgRow2:AddChildren(eStart, eEnd)
	sgRow3:AddChildren(bStart, bEnd)
	sgControls:AddChildren(bExport, bRemove)
end

function EmergeRT:DrawPlayers(widget)
	local sfPlayers = AceGUI:Create("ScrollFrame")
	sfPlayers:SetLayout("List")	

	local sgTree = AceGUI:Create("SimpleGroup")
	sgTree:SetLayout("Fill")
	sgTree:SetFullWidth(true)
	sgTree:SetHeight(250)
	
	table.sort(self.db.profile.events[selectedEvent].attendees, function(a,b) return strlower(a.name) < strlower (b.name) end)
	
	local treePlayers = AceGUI:Create("TreeGroup")
	treePlayers:SetLayout("List")
	local tree = {}
	for i,v in ipairs(self.db.profile.events[selectedEvent].attendees) do
		table.insert(tree, {value = i, text = v.name})
	end
	treePlayers:SetTree(tree)
	treePlayers:SetCallback("OnGroupSelected", function(w,e,v) self:SelectPlayer(w,v) end)

	local bNew = AceGUI:Create("Button")
	bNew:SetText("Add Player")
	bNew:SetWidth(200)
	bNew:SetCallback("OnClick", function()
									table.insert(self.db.profile.events[selectedEvent].attendees, {name="Player"})
									selectedPlayer = #self.db.profile.events[selectedEvent].attendees
									self:ShowGUI()
								end)
	
	widget:AddChild(sfPlayers)
	sfPlayers:AddChildren(bNew, sgTree)
	sgTree:AddChild(treePlayers)
	
	if selectedPlayer then treePlayers:Select(selectedPlayer) end
end

function EmergeRT:DrawJoins(widget)
	local sfJoins = AceGUI:Create("ScrollFrame")
	sfJoins:SetLayout("List")	

	local sgTree = AceGUI:Create("SimpleGroup")
	sgTree:SetLayout("Fill")
	sgTree:SetFullWidth(true)
	sgTree:SetHeight(250)
	
	local treeJoins = AceGUI:Create("TreeGroup")
	treeJoins:SetLayout("List")
	local tree = {}
	for i,v in ipairs(self.db.profile.events[selectedEvent].joins) do
		table.insert(tree, {value = i, text = v.name})
	end
	treeJoins:SetTree(tree)
	treeJoins:SetCallback("OnGroupSelected", function(w,e,v) self:SelectJoin(w,v) end)

	local bNew = AceGUI:Create("Button")
	bNew:SetText("Add Join")
	bNew:SetWidth(200)
	bNew:SetCallback("OnClick", function()
									table.insert(self.db.profile.events[selectedEvent].joins, {name="Player",time=time()})
									selectedJoin = #self.db.profile.events[selectedEvent].joins
									self:ShowGUI()
								end)
	
	widget:AddChild(sfJoins)
	sfJoins:AddChildren(bNew, sgTree)
	sgTree:AddChild(treeJoins)
	
	if selectedJoin then treeJoins:Select(selectedJoin) end
end

function EmergeRT:DrawLeaves(widget)
	local sfLeaves = AceGUI:Create("ScrollFrame")
	sfLeaves:SetLayout("List")	

	local sgTree = AceGUI:Create("SimpleGroup")
	sgTree:SetLayout("Fill")
	sgTree:SetFullWidth(true)
	sgTree:SetHeight(250)
	
	local treeLeaves = AceGUI:Create("TreeGroup")
	treeLeaves:SetLayout("List")
	local tree = {}
	for i,v in ipairs(self.db.profile.events[selectedEvent].leaves) do
		table.insert(tree, {value = i, text = v.name})
	end
	treeLeaves:SetTree(tree)
	treeLeaves:SetCallback("OnGroupSelected", function(w,e,v) self:SelectLeave(w,v) end)

	local bNew = AceGUI:Create("Button")
	bNew:SetText("Add Leave")
	bNew:SetWidth(200)
	bNew:SetCallback("OnClick", function()
									table.insert(self.db.profile.events[selectedEvent].leaves, {name="Player",time=time()})
									selectedLeave = #self.db.profile.events[selectedEvent].leaves
									self:ShowGUI()
								end)
	
	widget:AddChild(sfLeaves)
	sfLeaves:AddChildren(bNew, sgTree)
	sgTree:AddChild(treeLeaves)
	
	if selectedLeave then treeLeaves:Select(selectedLeave) end
end

function EmergeRT:DrawBosses(widget)
	local sfBosses = AceGUI:Create("ScrollFrame")
	sfBosses:SetLayout("List")	

	local sgTree = AceGUI:Create("SimpleGroup")
	sgTree:SetLayout("Fill")
	sgTree:SetFullWidth(true)
	sgTree:SetHeight(250)
	
	local treeBosses = AceGUI:Create("TreeGroup")
	treeBosses:SetLayout("List")
	local tree = {}
	for i,v in ipairs(self.db.profile.events[selectedEvent].bosses) do
		table.insert(tree, {value = i, text = v.name})
	end
	treeBosses:SetTree(tree)
	treeBosses:SetCallback("OnGroupSelected", function(w,e,v) self:SelectBoss(w,v) end)

	local bNew = AceGUI:Create("Button")
	bNew:SetText("Add Boss")
	bNew:SetWidth(200)
	bNew:SetCallback("OnClick", function()
									self:AddBoss()
									selectedBoss = #self.db.profile.events[selectedEvent].bosses
									self:ShowGUI()
								end)
	
	widget:AddChild(sfBosses)
	sfBosses:AddChildren(bNew, sgTree)
	sgTree:AddChild(treeBosses)
	
	if selectedBoss then treeBosses:Select(selectedBoss) end
end

function EmergeRT:DrawWipes(widget)
	local sfWipes = AceGUI:Create("ScrollFrame")
	sfWipes:SetLayout("List")	

	local sgTree = AceGUI:Create("SimpleGroup")
	sgTree:SetLayout("Fill")
	sgTree:SetFullWidth(true)
	sgTree:SetHeight(250)
	
	local treeWipes = AceGUI:Create("TreeGroup")
	treeWipes:SetLayout("List")
	local tree = {}
	for i,v in ipairs(self.db.profile.events[selectedEvent].wipes) do
		table.insert(tree, {value = i, text = FromUnixTime(v.time)})
	end
	treeWipes:SetTree(tree)
	treeWipes:SetCallback("OnGroupSelected", function(w,e,v) self:SelectWipe(w,v) end)

	local bNew = AceGUI:Create("Button")
	bNew:SetText("Add Wipe")
	bNew:SetWidth(200)
	bNew:SetCallback("OnClick", function()
									self:AddWipe()
									selectedWipe = #self.db.profile.events[selectedEvent].wipes
									self:ShowGUI()
								end)
	
	widget:AddChild(sfWipes)
	sfWipes:AddChildren(bNew, sgTree)
	sgTree:AddChild(treeWipes)
	
	if selectedWipe then treeWipes:Select(selectedWipe) end
end

function EmergeRT:DrawLoots(widget)
	local sfLoots = AceGUI:Create("ScrollFrame")
	sfLoots:SetLayout("List")	

	local sgTree = AceGUI:Create("SimpleGroup")
	sgTree:SetLayout("Fill")
	sgTree:SetFullWidth(true)
	sgTree:SetHeight(250)
	
	local treeLoots = AceGUI:Create("TreeGroup")
	treeLoots:SetLayout("Fill")
	local tree = {}
	for i,v in ipairs(self.db.profile.events[selectedEvent].loots) do
		table.insert(tree, {value = i, text = GetItemInfo(v.item)})
	end
	treeLoots:SetTree(tree)
	treeLoots:SetCallback("OnGroupSelected", function(w,e,v) self:SelectItem(w,v) end)

	local sgNew = AceGUI:Create("SimpleGroup")
	sgNew:SetLayout("Flow")
	sgNew:SetFullWidth(true)
	
	local eNew = AceGUI:Create("EditBox")
	eNew:SetLabel("New Item")
	eNew:SetText()
	eNew:SetWidth(200)
	eNew:SetCallback("OnEnterPressed", function(widget, event, text)
											itemID = text
											widget.editbox:ClearFocus()
										end)
	LinkBox = eNew

	local bNew = AceGUI:Create("Button")
	bNew:SetText("Add Item")
	bNew:SetWidth(200)
	bNew:SetCallback("OnClick", function()
									if not itemID then self:Print("No Item set!") return end
									_,itemID = GetItemInfo(itemID)
									if(itemID) then
										_, _, itemID = string.find(itemID, "^|c%x+|%a+:(%d+):.+|h%[.*%]")
										table.insert(self.db.profile.events[selectedEvent].loots, {['item']=tonumber(itemID),['player']="player",['dkp']=0,['time']=time(),['boss']="boss",['zone']=GetRealZoneText(),['difficulty']=GetRaidDifficulty()})
										selectedItem = #self.db.profile.events[selectedEvent].loots
										self:ShowGUI()
									else
										self:Print("Invalid Item: "..itemID)
									end
								end)
	
	widget:AddChild(sfLoots)
	sfLoots:AddChildren(sgNew, sgTree)
	sgNew:AddChildren(eNew, bNew)
	sgTree:AddChild(treeLoots)
	
	if selectedItem then treeLoots:Select(selectedItem) end
end

function EmergeRT:SelectOverview(widget, event, value)
	widget:ReleaseChildren()
	if selectedOverview ~= value then
		selectedOverview = value
		selectedItem = nil
	end
	if value == "Event" then
		self:DrawEvent(widget)
	elseif value == "Players" then
		self:DrawPlayers(widget)	
	elseif value == "Joins" then
		self:DrawJoins(widget)
	elseif value == "Leaves" then
		self:DrawLeaves(widget)
	elseif value == "Bosses" then
		self:DrawBosses(widget)
	elseif value == "Wipes" then
		self:DrawWipes(widget)
	elseif value == "Loots" then
		self:DrawLoots(widget)
	end
end

function EmergeRT:SelectPlayer(widget, value)
	widget:ReleaseChildren()
	if selectedPlayer ~= value then
		selectedPlayer = value
	end
	
	local sgRow1 = AceGUI:Create("SimpleGroup")
	sgRow1:SetLayout("Flow")
	sgRow1:SetFullWidth(true)
	
	local sgRow2 = AceGUI:Create("SimpleGroup")
	sgRow2:SetLayout("Flow")
	sgRow2:SetFullWidth(true)
	
	local ePlayer = AceGUI:Create("EditBox")
	ePlayer:SetLabel("Player")
	ePlayer:SetWidth(180)
	ePlayer:SetText(self.db.profile.events[selectedEvent].attendees[value].name)
	ePlayer:SetCallback("OnEnterPressed",function(widget,event,text) self.db.profile.events[selectedEvent].attendees[value].name = strtrim(text) self:ShowGUI() end )
	
	local eClass = AceGUI:Create("EditBox")
	eClass:SetLabel("Class")
	eClass:SetWidth(180)
	eClass:SetText(self.db.profile.events[selectedEvent].attendees[value].class)
	eClass:SetCallback("OnEnterPressed",function(widget,event,text) self.db.profile.events[selectedEvent].attendees[value].class = strtrim(text) end )
	
	local eLevel = AceGUI:Create("EditBox")
	eLevel:SetLabel("Level")
	eLevel:SetWidth(180)
	eLevel:SetText(self.db.profile.events[selectedEvent].attendees[value].level)
	eLevel:SetCallback("OnEnterPressed",function(widget,event,text) self.db.profile.events[selectedEvent].attendees[value].level = strtrim(text) end )
	
	local eRace = AceGUI:Create("EditBox")
	eRace:SetLabel("Race")
	eRace:SetWidth(180)
	eRace:SetText(self.db.profile.events[selectedEvent].attendees[value].race)
	eRace:SetCallback("OnEnterPressed",function(widget,event,text) self.db.profile.events[selectedEvent].attendees[value].race = strtrim(text) end )
	
	local eNote = AceGUI:Create("EditBox")
	eNote:SetLabel("Note")
	eNote:SetWidth(180)
	eNote:SetText(self.db.profile.events[selectedEvent].attendees[value].note)
	eNote:SetCallback("OnEnterPressed",function(widget,event,text) self.db.profile.events[selectedEvent].attendees[value].note = strtrim(text) end )	
	
	local bRemove = AceGUI:Create("Button")
	bRemove:SetText("Remove Player")
	bRemove:SetWidth(180)
	bRemove:SetCallback("OnClick", function()
								table.remove(self.db.profile.events[selectedEvent].attendees, value)
								selectedPlayer = nil
								self:ShowGUI()
							end )
	
	widget:AddChildren(sgRow1, sgRow2, eNote, bRemove)
	sgRow1:AddChildren(ePlayer, eLevel)
	sgRow2:AddChildren(eClass, eRace)
end

function EmergeRT:SelectJoin(widget, value)
	widget:ReleaseChildren()
	if selectedJoin ~= value then
		selectedJoin = value
	end
	
	local ePlayer = AceGUI:Create("EditBox")
	ePlayer:SetLabel("Player")
	ePlayer:SetWidth(180)
	ePlayer:SetText(self.db.profile.events[selectedEvent].joins[value].name)
	ePlayer:SetCallback("OnEnterPressed",function(widget,event,text) self.db.profile.events[selectedEvent].joins[value].name = strtrim(text) self:ShowGUI() end )
	
	local sgTime = AceGUI:Create("SimpleGroup")
	sgTime:SetLayout("Flow")
	sgTime:SetFullWidth(true)
	
	local eTime = AceGUI:Create("EditBox")
	eTime:SetLabel("Time")
	eTime:SetWidth(180)
	eTime:SetText(FromUnixTime(self.db.profile.events[selectedEvent].joins[value].time))
	eTime:SetCallback("OnEnterPressed",function(widget,event,text) self.db.profile.events[selectedEvent].joins[value].time = TimeAsUnix(strtrim(text)) end )
	
	local bTime = AceGUI:Create("Button")
	bTime:SetText("Current Time")
	bTime:SetWidth(180)
	bTime:SetCallback("OnClick", function() local t = time() eTime:SetText(FromUnixTime(t)) self.db.profile.events[selectedEvent].joins[value].time = t end)
	
	local bRemove = AceGUI:Create("Button")
	bRemove:SetText("Remove Join")
	bRemove:SetWidth(180)
	bRemove:SetCallback("OnClick", function()
								table.remove(self.db.profile.events[selectedEvent].joins, value)
								selectedJoin = nil
								self:ShowGUI()
							end )
	
	widget:AddChildren(ePlayer, sgTime, bRemove)
	sgTime:AddChildren(eTime, bTime)
end

function EmergeRT:SelectLeave(widget, value)
	widget:ReleaseChildren()
	if selectedLeave ~= value then
		selectedLeave = value
	end
	
	local ePlayer = AceGUI:Create("EditBox")
	ePlayer:SetLabel("Player")
	ePlayer:SetWidth(180)
	ePlayer:SetText(self.db.profile.events[selectedEvent].leaves[value].name)
	ePlayer:SetCallback("OnEnterPressed",function(widget,event,text) self.db.profile.events[selectedEvent].leaves[value].name = strtrim(text) self:ShowGUI() end )
	
	local sgTime = AceGUI:Create("SimpleGroup")
	sgTime:SetLayout("Flow")
	sgTime:SetFullWidth(true)
	
	local eTime = AceGUI:Create("EditBox")
	eTime:SetLabel("Time")
	eTime:SetWidth(180)
	eTime:SetText(FromUnixTime(self.db.profile.events[selectedEvent].leaves[value].time))
	eTime:SetCallback("OnEnterPressed",function(widget,event,text) self.db.profile.events[selectedEvent].leaves[value].time = TimeAsUnix(strtrim(text)) end )
	
	local bTime = AceGUI:Create("Button")
	bTime:SetText("Current Time")
	bTime:SetWidth(180)
	bTime:SetCallback("OnClick", function() local t = time() eTime:SetText(FromUnixTime(t)) self.db.profile.events[selectedEvent].leaves[value].time = t end)
	
	local bRemove = AceGUI:Create("Button")
	bRemove:SetText("Remove Leave")
	bRemove:SetWidth(180)
	bRemove:SetCallback("OnClick", function()
								table.remove(self.db.profile.events[selectedEvent].leaves, value)
								selectedLeave = nil
								self:ShowGUI()
							end )
	
	widget:AddChildren(ePlayer, sgTime, bRemove)
	sgTime:AddChildren(eTime, bTime)
end

function EmergeRT:SelectBoss(widget, value)
	widget:ReleaseChildren()
	if selectedBoss ~= value then
		selectedBoss = value
	end
	
	local eBoss = AceGUI:Create("EditBox")
	eBoss:SetLabel("Boss")
	eBoss:SetWidth(180)
	eBoss:SetText(self.db.profile.events[selectedEvent].bosses[value].name)
	eBoss:SetCallback("OnEnterPressed",function(widget,event,text) self.db.profile.events[selectedEvent].bosses[value].name = strtrim(text) self:ShowGUI() end )
	
	local sgTime = AceGUI:Create("SimpleGroup")
	sgTime:SetLayout("Flow")
	sgTime:SetFullWidth(true)
	
	local eTime = AceGUI:Create("EditBox")
	eTime:SetLabel("Time")
	eTime:SetWidth(180)
	eTime:SetText(FromUnixTime(self.db.profile.events[selectedEvent].bosses[value].time))
	eTime:SetCallback("OnEnterPressed",function(widget,event,text) self.db.profile.events[selectedEvent].bosses[value].time = TimeAsUnix(strtrim(text)) end )
	
	local bTime = AceGUI:Create("Button")
	bTime:SetText("Current Time")
	bTime:SetWidth(180)
	bTime:SetCallback("OnClick", function() local t = time() eTime:SetText(FromUnixTime(t)) self.db.profile.events[selectedEvent].bosses[value].time = t end)
	
	local bRemove = AceGUI:Create("Button")
	bRemove:SetText("Remove Boss")
	bRemove:SetWidth(180)
	bRemove:SetCallback("OnClick", function()
								table.remove(self.db.profile.events[selectedEvent].bosses, value)
								selectedBoss = nil
								self:ShowGUI()
							end )
	
	widget:AddChildren(eBoss, sgTime, bRemove)
	sgTime:AddChildren(eTime, bTime)
end

function EmergeRT:SelectWipe(widget, value)
	widget:ReleaseChildren()
	if selectedWipe ~= value then
		selectedWipe = value
	end
	
	local sgTime = AceGUI:Create("SimpleGroup")
	sgTime:SetLayout("Flow")
	sgTime:SetFullWidth(true)
	
	local eTime = AceGUI:Create("EditBox")
	eTime:SetLabel("Time")
	eTime:SetWidth(180)
	eTime:SetText(FromUnixTime(self.db.profile.events[selectedEvent].wipes[value].time))
	eTime:SetCallback("OnEnterPressed",function(widget,event,text) self.db.profile.events[selectedEvent].wipes[value].time = TimeAsUnix(strtrim(text)) self:ShowGUI() end )
	
	local bTime = AceGUI:Create("Button")
	bTime:SetText("Current Time")
	bTime:SetWidth(180)
	bTime:SetCallback("OnClick", function() local t = time() eTime:SetText(FromUnixTime(t)) self.db.profile.events[selectedEvent].wipes[value].time = t end)
	
	local bRemove = AceGUI:Create("Button")
	bRemove:SetText("Remove Wipe")
	bRemove:SetWidth(180)
	bRemove:SetCallback("OnClick", function()
								table.remove(self.db.profile.events[selectedEvent].wipes, value)
								selectedWipe = nil
								self:ShowGUI()
							end )
	
	widget:AddChildren(sgTime, bRemove)
	sgTime:AddChildren(eTime, bTime)
end

function EmergeRT:SelectItem(widget, value)
	widget:ReleaseChildren()
	if selectedItem ~= value then
		selectedItem = value
	end
	
	local sf = AceGUI:Create("ScrollFrame")
	sf:SetLayout("List")
	
	local sgRow1 = AceGUI:Create("SimpleGroup")
	sgRow1:SetLayout("Flow")
	sgRow1:SetFullWidth(true)
	
	local sgRow2 = AceGUI:Create("SimpleGroup")
	sgRow2:SetLayout("Flow")
	sgRow2:SetFullWidth(true)
	
	local eItemID = AceGUI:Create("EditBox")
	eItemID:SetLabel("ItemID")
	eItemID:SetWidth(180)
	eItemID:SetText(self.db.profile.events[selectedEvent].loots[value].item)
	eItemID:SetCallback("OnEnterPressed",function(widget,event,text) widget.editbox:ClearFocus() self.db.profile.events[selectedEvent].loots[value].item = strtrim(text) end )
	
	local eLooter = AceGUI:Create("EditBox")
	eLooter:SetLabel("Looted by")
	eLooter:SetWidth(180)
	eLooter:SetText(self.db.profile.events[selectedEvent].loots[value].player)
	eLooter:SetCallback("OnEnterPressed",function(widget,event,text) widget.editbox:ClearFocus() self.db.profile.events[selectedEvent].loots[value].player = strtrim(text) end )
	
	local eDKP = AceGUI:Create("EditBox")
	eDKP:SetLabel("DKP")
	eDKP:SetWidth(180)
	eDKP:SetText(self.db.profile.events[selectedEvent].loots[value].dkp)
	eDKP:SetCallback("OnEnterPressed",function(widget,event,text) widget.editbox:ClearFocus() self.db.profile.events[selectedEvent].loots[value].dkp = strtrim(text) end )
	
	local sgTime = AceGUI:Create("SimpleGroup")
	sgTime:SetFullWidth(true)
	sgTime:SetLayout("Flow")
	
	local eTime = AceGUI:Create("EditBox")
	eTime:SetLabel("Time")
	eTime:SetWidth(180)
	eTime:SetText(FromUnixTime(self.db.profile.events[selectedEvent].loots[value].time))
	eTime:SetCallback("OnEnterPressed",function(widget,event,text) self.db.profile.events[selectedEvent].loots[value].time = TimeAsUnix(strtrim(text)) end )
	
	local bTime = AceGUI:Create("Button")
	bTime:SetText("Current Time")
	bTime:SetWidth(180)
	bTime:SetCallback("OnClick", function() local t = time() eTime:SetText(FromUnixTime(t)) self.db.profile.events[selectedEvent].loots[value].time = t end)
	
	local eZone = AceGUI:Create("EditBox")
	eZone:SetLabel("Zone")
	eZone:SetWidth(180)
	eZone:SetText(self.db.profile.events[selectedEvent].loots[value].zone)
	eZone:SetCallback("OnEnterPressed",function(widget,event,text) widget.editbox:ClearFocus() self.db.profile.events[selectedEvent].loots[value].zone = strtrim(text) end )
	
	local eBoss = AceGUI:Create("EditBox")
	eBoss:SetLabel("Boss")
	eBoss:SetWidth(180)
	eBoss:SetText(self.db.profile.events[selectedEvent].loots[value].boss)
	eBoss:SetCallback("OnEnterPressed",function(widget,event,text) widget.editbox:ClearFocus() self.db.profile.events[selectedEvent].loots[value].boss = strtrim(text) end )
	
	local eDifficulty = AceGUI:Create("EditBox")
	eDifficulty:SetLabel("Difficulty")
	eDifficulty:SetWidth(180)
	eDifficulty:SetText(self.db.profile.events[selectedEvent].loots[value].difficulty)
	eDifficulty:SetCallback("OnEnterPressed",function(widget,event,text) widget.editbox:ClearFocus() self.db.profile.events[selectedEvent].loots[value].difficulty = tonumber(strtrim(text)) end )
	
	local bRemove = AceGUI:Create("Button")
	bRemove:SetText("Remove Item")
	bRemove:SetWidth(180)
	bRemove:SetCallback("OnClick", function()
								table.remove(self.db.profile.events[selectedEvent].loots, value)
								selectedItem = nil
								self:ShowGUI()
							end )
	widget:AddChild(sf)
	sf:AddChildren(sgRow1, eDKP, sgTime, sgRow2, eDifficulty, bRemove)
	sgRow1:AddChildren(eItemID, eLooter)
	sgTime:AddChildren(eTime, bTime)
	sgRow2:AddChildren(eZone, eBoss)
end

function EmergeRT:GetDKP()
	local lootid = #self.db.profile.events[#self.db.profile.events].loots
	local item = GetItemInfo(self.db.profile.events[#self.db.profile.events].loots[lootid].item)
	local player = self.db.profile.events[#self.db.profile.events].loots[lootid].player
	
	local fDKP = AceGUI:Create("BlankFrame")
	fDKP:SetTitle("DKP eingeben")
	fDKP:SetLayout("List")
	fDKP:SetWidth(400)
	fDKP:SetHeight(200)
	fDKP:SetCallback("OnClose", function(widget)
								AceGUI:Release(widget) end)
	fDKP:Show()

	
	local lLooter = AceGUI:Create("Label")
	lLooter:SetWidth(300)
	lLooter:SetText(player..": "..item)
	
	local eDKP = AceGUI:Create("EditBox")
	eDKP:SetLabel("DKP:")
	eDKP:SetWidth(300)
	eDKP:SetText("0")
	eDKP:SetCallback("OnEnterPressed",function(widget,event,text)
										widget.editbox:ClearFocus()
										self.db.profile.events[#self.db.profile.events].loots[lootid].dkp = strtrim(text)
									end )
	
	local sgControls = AceGUI:Create("SimpleGroup")
	sgControls:SetLayout("Flow")
	sgControls:SetFullWidth(true)
	
	local sgDKP = AceGUI:Create("SimpleGroup")
	sgDKP:SetLayout("Flow")
	sgDKP:SetFullWidth(true)
	
	local bDisenchanted = AceGUI:Create("Button")
	bDisenchanted:SetText("disenchanted")
	bDisenchanted:SetWidth(200)
	bDisenchanted:SetCallback("OnClick", function()
								self.db.profile.events[#self.db.profile.events].loots[lootid].player = "disenchanted"
								lLooter:SetText("disenchanted: "..item)
							end )
	local bBank = AceGUI:Create("Button")
	bBank:SetText("bank")
	bBank:SetWidth(200)
	bBank:SetCallback("OnClick", function()
								self.db.profile.events[#self.db.profile.events].loots[lootid].player = "bank"
								lLooter:SetText("bank: "..item)
							end )
	
	bYes = AceGUI:Create("Button")
	bYes:SetText("Subtract DKP")
	bYes:SetWidth(200)
	bYes:SetCallback("OnClick", function()
								if _G["GDKP_DKP_Sub"] and self.db.profile.events[#self.db.profile.events].dkpSet then
									GDKP_DKP_Sub(self.db.profile.events[#self.db.profile.events].loots[lootid].player.." "..self.db.profile.events[#self.db.profile.events].loots[lootid].dkp.." "..self.db.profile.events[#self.db.profile.events].dkpSet)
								end
								fDKP:Hide()
							end)
	
	bNo = AceGUI:Create("Button")
	bNo:SetText("Close")
	bNo:SetWidth(200)
	bNo:SetCallback("OnClick", function()
								fDKP:Hide()
							end)
	

	
	fDKP:AddChildren(lLooter, eDKP, sgDKP, sgControls)
	sgDKP:AddChildren(bDisenchanted, bBank)
	sgControls:AddChildren(bYes, bNo)
end

function EmergeRT:IsWipe()
	fWipe = AceGUI:Create("BlankFrame")
	fWipe:SetTitle("WipeCheck")
	fWipe:SetLayout("List")
	fWipe:SetWidth(200)
	fWipe:SetHeight(100)
	fWipe:SetCallback("OnClose", function(widget)
									checkPending = false
									AceGUI:Release(widget)
								end)
	fWipe:Show()
	
	lQuestion = AceGUI:Create("Label")
	lQuestion:SetWidth(150)
	lQuestion:SetText("Wiped?")
	
	sgControls = AceGUI:Create("SimpleGroup")
	sgControls:SetLayout("Flow")
	
	bYes = AceGUI:Create("Button")
	bYes:SetText("Yes")
	bYes:SetWidth(50)
	bYes:SetCallback("OnClick", function()
								self:AddWipe()
								fWipe:Hide()
							end)
	
	bNo = AceGUI:Create("Button")
	bNo:SetText("No")
	bNo:SetWidth(50)
	bNo:SetCallback("OnClick", function()
								fWipe:Hide()
							end)
							
	fWipe:AddChildren(lQuestion, sgControls)
	sgControls:AddChildren(bYes, bNo)
end

function EmergeRT:ClearCheck()
	fReset = AceGUI:Create("BlankFrame")
	fReset:SetTitle("Reset")
	fReset:SetLayout("List")
	fReset:SetWidth(200)
	fReset:SetHeight(100)
	fReset:SetCallback("OnClose", function(widget) AceGUI:Release(widget) end)
	fReset:Show()
	
	lQuestion = AceGUI:Create("Label")
	lQuestion:SetWidth(150)
	lQuestion:SetText("Wirklich alle Daten l\195\182schen?")
	
	sgControls = AceGUI:Create("SimpleGroup")
	sgControls:SetLayout("Flow")
	
	bYes = AceGUI:Create("Button")
	bYes:SetText("Yes")
	bYes:SetWidth(50)
	bYes:SetCallback("OnClick", function()
								self:ClearAll()
								fReset:Hide()
							end)
	
	bNo = AceGUI:Create("Button")
	bNo:SetText("No")
	bNo:SetWidth(50)
	bNo:SetCallback("OnClick", function()
								fReset:Hide()
							end)
	
	fReset:AddChildren(lQuestion, sgControls)
	sgControls:AddChildren(bYes, bNo)
end
